#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "index.h"
#include <asm/fcntl.h>

static int errno;

static int reindex (int fd, int indexfd, time_t new_mtime) {
    int err;
    int delim_pos = 0, pos = 0, fortunes = 0, done = 0;
    char ch = -1;

    lseek (fd, 0, 0);
    /*skip header*/
    if (lseek (indexfd, INDEX_HEADER_END, 0) != INDEX_HEADER_END) {
	return -1;
    }

    err = write (indexfd, (void *) &pos, sizeof (int)); /* start of first fortune*/
    if (err != sizeof (int)) return -errno;

    while (!done) {
	err = read (fd, &ch, 1);
	if (err == 1) {
	    pos++;
	    if (ch == delimiter [delim_pos]) {
		delim_pos++;
		if (delim_pos == delim_size) {
		    /* mark hit */
		    err = write (indexfd, (void *) &pos, sizeof (int));
		    if (err != sizeof (int)) return -errno;
		    fortunes++;
		    delim_pos = 0;
		}
	    } else {
		if (delim_pos) {
		    if (ch == *delimiter) {
			delim_pos = 1;
		    } else {
			delim_pos = 0;
		    }
		}
	    }
	} else {
	    done = 1;
	} 
    }

    if (err == -1) return -errno;
    lseek (indexfd, 0, 0);
    /* write new header */
    err = write (indexfd, (void *) &fortunes, sizeof (int));
    if (err == -1) return -errno;
    err = write (indexfd, (void *) &new_mtime, sizeof (time_t));
    if (err == -1) return -errno;
    return 0;
}

int main (int argc, char **argv) {
    int fd, indexfd, fnlen, err;
    char *fortune_file, *index_file;
    struct stat statbuf;

    if (argc != 2) {
	printf ("Error: Need argument of file to index\n");
	exit (1);
    }

    fortune_file = argv [1];
    fnlen = strlen (fortune_file);
    index_file = malloc (fnlen + 7);
    strcpy (index_file, fortune_file);
    strcpy (index_file + fnlen, ".index");

    fd = open (fortune_file, O_RDONLY, 0);
    if (!fd) {
	perror ("Can't open fortune file for reading");
	exit (1);
    }
    indexfd = open (index_file, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
    if (!indexfd) {
	perror ("Can't open index file for read/write");
	exit (1);
    }
    if (fstat (fd, &statbuf) == -1) {
	perror ("Couldn't stat fortune file");
	exit (1);
    }
    err = reindex (fd, indexfd, statbuf.st_mtime);
    if (err) {
	perror ("Can't index");
	exit (1);
    }
    close (fd);
    close (indexfd);
}
